// server for mico

#include "../config.h"

#include <stdio.h>
#include <CORBA.h>
#ifdef USE_NAME_SERVICE
  #include <mico/CosNaming.h>
#endif

#include "counter.h"
#include "../misc.h"

////////////////////////////////////////////////////////////////////////
// CounterImpl

class CounterImpl: public virtual POA_Counter
{
protected:
    short count_;
public:
    CounterImpl(): count_(0) { }
    virtual ~CounterImpl() { }
    
    virtual CORBA::Short value() { return count_; }
    virtual void incr() { count_++; }
    virtual void decr() { count_--; }
    virtual char* name() { return CORBA::string_dup("Counter"); }
protected:
    CounterImpl(short v): count_(v) { }
};

////////////////////////////////////////////////////////////////////////
// ReverseCounterImpl

class ReverseCounterImpl: public virtual POA_ReverseCounter,
                        public virtual CounterImpl
{
    typedef CounterImpl super;
public:
    ReverseCounterImpl(): super(1000) { }
    virtual ~ReverseCounterImpl() { }
    
    virtual void incr() { count_--; }
    virtual void decr() { count_++; }
    virtual char* name() { return CORBA::string_dup("ReverseCounter"); }
};

////////////////////////////////////////////////////////////////////////

#ifndef USE_NAME_SERVICE
static const char* ior_file = "../counter.ior";
#endif

int main(int argc, char* argv[])
{
    // Initialise the ORB.
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
    
    // Obtain a reference to the root POA.
    CORBA::Object_var poaObj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow(poaObj);

    // We allocate the objects on the heap.  Since these are reference
    // counted objects, they will be deleted by the POA when they are no
    // longer needed.
    CounterImpl* myCounter = new CounterImpl();
    ReverseCounterImpl* myReverseCounter = new ReverseCounterImpl();

    // Activate the objects.  This tells the POA that the objects are
    // ready to accept requests.
    PortableServer::ObjectId_var myCounter_iid
        = poa->activate_object(myCounter);
    PortableServer::ObjectId_var myReverseCounter_iid
        = poa->activate_object(myReverseCounter);

#ifdef USE_NAME_SERVICE
    CORBA::Object_var nsobj
                = orb->resolve_initial_references("NameService");
    if (CORBA::is_nil(nsobj)) {
        printf("resolve NameService failed.\n");
        return 1;
    }
    CosNaming::NamingContext_var naming
                            = CosNaming::NamingContext::_narrow(nsobj);

    CosNaming::Name name;
    name.length(1);
    name[0].id = CORBA::string_dup(COUNTER_NAME);
    name[0].kind = CORBA::string_dup("");
    naming->rebind(name, nc->_this());

    name.length(1);
    name[0].id = CORBA::string_dup(REVERSE_NAME);
    name[0].kind = CORBA::string_dup("");
    naming->rebind(name, rc->_this());
#else
    // Obtain a reference to each object and output the stringified
    // IOR to ior-file
    FILE* fp = fopen(ior_file, "w");
    if (!fp) {
        perror("ior-file open failed");
        return 1;
    }
    CORBA::String_var sior;
    sior = orb->object_to_string(myCounter->_this());
    fprintf(fp, "%s\n", (const char*) sior);
    sior = orb->object_to_string(myReverseCounter->_this());
    fprintf(fp, "%s\n", (const char*) sior);
    fclose(fp);
#endif  // USE_NAME_SERVICE
    // Obtain a POAManager, and tell the POA to start accepting
    // requests on its objects.
    PortableServer::POAManager_var poaMgr = poa->the_POAManager();
    poaMgr->activate();

    printf("ready.\n");
    orb->run();
    
    poa->destroy(true, true);
    delete myCounter;
    delete myReverseCounter;
#ifndef USE_NAME_SERVICE
    remove(ior_file);
#endif

    return 0;
}
